#include <stdio.h>
#include <stdlib.h>
#include "piece.h"
#include "game.h"

#define LENGTH 6

struct game_s {
  piece *tabPiece;
  char **board;
  int nb_pieces;
  int nb_moves;
  bool win;
};

struct cgame_s {
  game *game;
};

void display(cgame g) {
  for (int j = 5; j >= 0;  -- j) {
    printf("-------------------------\n"); //au dessus des cases
    for(int i = 0; i < LENGTH; ++ i) {
      if(g->board[i][j] >= 0 && g->board[i][j] <= g->nb_pieces) //si une piece est dans la case
	printf("| %d ", g->board[i][j]);
      else //si la case est vide
 	printf("| %c ", g->board[i][j]);
    }
    printf("|\n");
  }
  printf("-------------------------\n"); //pour la dernière ligne
}

void init(game g) {
  for(int i = 0; i < LENGTH; i++) {
    for(int j = 0; j < LENGTH; j++)
      g->board[i][j]= ' '; //initialisation à espace dans les cases
  }
}

void printBoard(game g) {
  init(g); //on rafraîchit la grille
  int nbpi = g->nb_pieces;
  for(int i = 0; i < nbpi; ++ i) { //affecte à la grille les numéros des pieces
    int x = get_x(g->tabPiece[i]);
    int y = get_y(g->tabPiece[i]);
    bool hori = is_horizontal(g->tabPiece[i]);
    int w = get_width(g->tabPiece[i]);
    int h = get_height(g->tabPiece[i]);
    g->board[x][y] = i;
    if(hori) {
      g->board[x+1][y] = i;
      if(w == 3)
	g->board[x+2][y] = i;
    }
    else {
      g->board[x][y+1] = i;
      if(h == 3)
	g->board[x][y+2] = i;
    }
  }
  display(g); //affichage du nouvel état de la grille
}
      
game new_game_hr(int nb_pieces, piece *pieces) {
  game g;

  g = malloc(sizeof(*g));  //on alloue une game
  g->tabPiece = malloc(sizeof(piece) * nb_pieces); //on alloue et initialise le tableau des pieces
  for(int i = 0; i < nb_pieces; ++ i) {
    g->tabPiece[i] = new_piece_rh(0, 0, true, true);
    copy_piece(pieces[i],g->tabPiece[i]);
  }
  //on initialise la grille de jeu
  g->board = malloc(LENGTH * sizeof(int*));
  for(int i = 0; i < LENGTH; ++ i)
    g->board[i] = malloc(LENGTH * sizeof(int**));
  init(g);
  g->nb_moves = 0;
  g->nb_pieces = nb_pieces;
  g->win = false;
  return g;
}

void delete_game(game g) {
  //la grille est effacée
  for(int i = 0; i < LENGTH; ++ i) 
    free(g->board[i]);
  free(g->board);
  //les pieces sont supprimées
  for(int k = 0; k < g->nb_pieces; ++ k)
    delete_piece(g->tabPiece[k]);
  free(g->tabPiece);
  free(g);
}

void copy_game(cgame src, game dst) {
  //copie de l'état de la grille
  for(int i = 0; i < LENGTH; ++ i) {
    for(int j = 0; j < LENGTH; ++ j)
      dst->board[i][j] = src->board[i][j];
  }
  //copie des pieces 
  for(int k = 0; k < src->nb_pieces; ++k)
    dst->tabPiece[k] = src->tabPiece[k];
  dst->nb_pieces = src->nb_pieces;
  dst->nb_moves = src->nb_moves;
  dst->win = src->win;
}

int game_nb_pieces(cgame g) {
  return g->nb_pieces;
}

int game_nb_moves(cgame g) {
  return g->nb_moves;
}

cpiece game_piece(cgame g, int piece_num) {
  int nb_piece = game_nb_pieces(g);
  if(piece_num < 0 || piece_num > nb_piece) {
    fprintf(stderr, "La piece demandee n'existe pas");
    return NULL;
  }
  return g->tabPiece[piece_num];
}

bool game_over_hr(cgame g) {
  // si la piece 0 n'est pas en coordonnees (5,3)
  if (!((get_x(g->tabPiece[0]) + get_width(g->tabPiece[0]) - 1) == 5) && get_y(g->tabPiece[0]) == 3) 
    return false;
  printf("Congrats, you win this game !\n");
  return true;
}

bool play_move(game g, int piece_num, dir d, int distance) {
 if(piece_num < 0 || piece_num > game_nb_pieces(g)) {
    fprintf(stderr, "La piece ciblée n'existe pas");
    return false;
 }
 bool isHorizontal = is_horizontal(g->tabPiece[piece_num]);
 int nbPieces = game_nb_pieces(g);
 //le mouvement n'est pas correct
 if((isHorizontal == true && (d == UP || d == DOWN)) || (isHorizontal == false && (d == LEFT || d == RIGHT))) //si la direction n'est pas correcte
   return false;
 if(isHorizontal == false) { // si la piece dépasse les limites verticales de la grille
   if(d == UP && (get_y(g->tabPiece[piece_num]) + (get_height(g->tabPiece[piece_num]) - 1) + distance > 5))
     return false;
   if(d == DOWN && (get_y(g->tabPiece[piece_num]) - distance < 0))
     return false;
 }
 if(isHorizontal == true) { // si la piece dépasse les limites horizontales de la grille
   if(d == LEFT && (get_x(g->tabPiece[piece_num]) - distance < 0))  
     return false;
   if(d == RIGHT && (get_x(g->tabPiece[piece_num]) + (get_width(g->tabPiece[piece_num]) - 1) + distance > 5))
     return false;
 }
 //si une autre piece bloque le mouvement
 piece piece_clone = new_piece_rh(0, 0, true, true);
 copy_piece(g->tabPiece[piece_num], piece_clone); //on crée une copie de notre piece
 for(int j = 0; j < nbPieces; ++ j) {
   if(j != piece_num) {
     int i = 1; //1 qui est la distance minimale d'un movement
     while(i < distance) {
       move_piece(piece_clone, d , i);
       if(intersect(piece_clone, g->tabPiece[j])) //test des intersections pas à pas
	 return false;
       i++;
     }
   }
 }
 //on supprime le clone crée
 delete_piece(piece_clone);
  //le mouvement est donc correct
   move_piece(g->tabPiece[piece_num], d ,distance);
   g->nb_moves += distance;
 return true;
}


