#include <stdlib.h>
#include <stdio.h>
#include "piece.h"
#include "game.h"

#define NB_PIECES 2
#define LENGTH 6

/**
 * @brief test if value is equal to expected; if not, displays an error message containing msg to standard error output 
 * @return true if expected==value
 */
bool test_equality_int(int expected, int value, char * msg) {
  if (value != expected)
    fprintf(stderr,"ERR: value expected %d compare to value computed %d. %s\n", expected, value, msg);
  return value == expected;
}

/**
 * @brief test if value is equal to expected; if not, displays an error message containing msg to standard error output 
 * @return true if expected==value
 */
bool test_equality_char(char expected, char value, char * msg) {
  if (value != expected)
    fprintf(stderr,"ERR: value expected %d compare to value computed %d. %s\n", expected, value, msg);
  return value == expected;
}

/**
 * @brief test if value is equal to expected; if not, displays an error message containing msg to standard error output 
 * @return true if expected==value
 */
bool test_equality_bool(bool expected, bool value, char * msg) {
  if(value != expected)
    fprintf(stderr,"ERR: value expected %d compare to value computed %d. %s\n", expected, value, msg);
  return value == expected;
}

piece pieces[NB_PIECES];
/*test config
...1..
...1..
.001..
......
......
......
*/

void create_tab_piece() {
  //v0 en (1,3) d'une taille de 2 cases en position horizontale
  pieces[0]= new_piece_rh(1,3, true, true);
  //v1 en (3,3) d'une taille de 3 cases en position verticale
  pieces[1] = new_piece_rh(3,3,false, false);
}

void delete_tab_piece() {
  delete_piece(pieces[0]);
  delete_piece(pieces[1]);
}

bool test_new_game_hr() {
  bool result = true;
  create_tab_piece();
  game g = new_game_hr(NB_PIECES, pieces);
  
  result = result && test_equality_char(3, get_y(game_piece(g,0)), "v0 en y = 3?");
  result = result && test_equality_char(1, get_x(game_piece(g,0)), "v0 en x = 1?");
  result = result && test_equality_char(3, get_y(game_piece(g,1)), "v1 en y = 3?");
  result = result && test_equality_char(3, get_x(game_piece(g,1)), "v1 en x = 3?");
  result = result && test_equality_int(NB_PIECES, game_nb_pieces(g), "game_nb_piece");
  result = result && test_equality_int(0, game_nb_moves(g), "game_nb_moves");
  result = result && test_equality_bool(false, game_over_hr(g), "game_over_hr");
  
  delete_game(g);
  delete_tab_piece();
  return result;
}

bool  test_copy_game() {
  bool result = true;
  create_tab_piece();
  game src = new_game_hr(NB_PIECES, pieces);
  
  piece pieceTMP[NB_PIECES];
  pieceTMP[0] = new_piece_rh(3,3,false, false);
  pieceTMP[1] = new_piece_rh(1,3, true, true);
  game dst = new_game_hr(NB_PIECES, pieceTMP);
  
  copy_game(src,dst);
  
  result = result && test_equality_char(get_x(game_piece(src,0)), get_x(game_piece(dst,0)), "coordonées de v0 dans les deux tableaux identiques ?");
  result = result && test_equality_char(get_y(game_piece(src,0)), get_y(game_piece(dst,0)), "coordonées de v0 dans les deux tableaux identiques ?");
  result = result && test_equality_int(game_nb_pieces(src),game_nb_pieces(dst), "game_nb_piece");
  result = result && test_equality_int(game_nb_moves(src), game_nb_moves(dst), "game_nb_moves");
  result = result && test_equality_bool(game_over_hr(src), game_over_hr(dst), "game_over_hr");
  
  delete_game(src);
  delete_game(dst);
  delete_piece(pieceTMP[0]);
  delete_piece(pieceTMP[1]);
  delete_tab_piece();
  return result;
}

bool test_play_move() {
  bool result = true;
  create_tab_piece();
  game g = new_game_hr(NB_PIECES, pieces);
  dir d;
  int piece_num;
  int distance;
  
  //déplacement de 3 cases de v1 vers le bas
  d = DOWN;
  piece_num = 1;
  distance = 3;
  
  result = result && test_equality_bool(true, play_move(g ,piece_num, d, distance), "v1 3 cases vers le bas");
  
  //déplacement de 3 cases de v0 vers la droite
  d = RIGHT;
  piece_num = 0;
  distance = 3; 
  
  result = result && test_equality_bool(true, play_move(g, piece_num, d, distance), "v0 3 cases vers la droite");
  result = result && test_equality_bool(true, game_over_hr(g), "game_over_hr");
  
  delete_game(g);
  delete_tab_piece();
  return result;
}

int main(int argc, char *argv[]) {
  bool result = true;
  
  result = result && test_equality_bool(true, test_new_game_hr(), "new_game_hr");
  result = result && test_equality_bool(true, test_copy_game(), "copy_game");
  result = result && test_equality_bool(true, test_play_move(), "play_move");

  if(result) {
    printf("All tests are good ! \n");
    return EXIT_SUCCESS;
  }
  else
    return EXIT_FAILURE;
}
